/*
	AvrXBufferedSerial.c

	Sample code for fully buffered interrupt driven serial I/O for the
	AVR processor.  Uses the AvrXFifo facility.

	Author: Larry Barello (larry@barello.net)

	Revision History:
	09-13-2005	- Initial version

	pitschu:	changes for AVR; added my own put/get functions

 */

//------------------------------------------------------------------------------
#include <inttypes.h>
#include <string.h>
#include <math.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#define _AVRXSERIALIO_C_
#include "AvrXSerialIo.h"
#include "main.h"
#include "stdio.h"

AVRX_DECL_FIFO(Rx0Buf, RX0_BUFSZ);
AVRX_DECL_FIFO(Tx0Buf, TX0_BUFSZ);

FILE* uart_filehandle;



int put_stdout(char c, FILE *f)	// blocking output
{
	return put_char0(c);
}



void InitSerial0(uint16_t ubrr)
{
	DDRD |= (1<<PD1);		// TX
	DDRD &= ~(1<<PD0);		// RX
	PORTD |= (1<<PD0);		// RX pullup ON

	UCSR0A = _B0(U2X0);
	UCSR0B = _B0(RXCIE0) | _B1(RXEN0) | _B1(TXEN0) | _B0(TXCIE0);
	UCSR0C = _B0(UMSEL01) | _B0(UMSEL00) |
			_B0(UPM01) | _B0(UPM00) |
			_B0(USBS0) |
			_B0(UCSZ02) | _B1(UCSZ01) | _B1(UCSZ00) |
			_B0(UCPOL0);
	UBRR0L = ubrr;
	UBRR0H = (uint8_t)(ubrr>>8);

	AVRX_INIT_FIFO(Rx0Buf);
	AVRX_INIT_FIFO(Tx0Buf);

	volatile char c = UDR0;
	UCSR0B |= (1<<TXCIE0);		// enabe TX and RX irqs
	UCSR0B |= (1<<RXCIE0);

	uart_filehandle = fdevopen (put_stdout, NULL);		// connect UART-TX to stdin/stderr
}




int put_c0(char c)	// Non blocking output
{
	int retc;
	retc = AvrXPutFifo(Tx0Buf, c);
	if (UCSR0A & (1<<UDRE0))
	{
		int c = AvrXPullFifo(Tx0Buf);	// Return -1 if empty (enables interrupts)
		if (c >= 0)						// Tricky tight code: only checking sign
			UDR0 = c;
	}
	return retc;
}




int put_char0( char c)	// Blocking output
{
	while (put_c0(c) == FIFO_ERR)
	{
		uint8_t oldSREG = SREG; // Save and restore the interrupt enable bit
		cli();
		if (UCSR0A & (1<<UDRE0))
		{
			int c = AvrXPullFifo(Tx0Buf);	// Return -1 if empty (enables interrupts)
			if (c >= 0)						// Tricky tight code: only checking sign
				UDR0 = c;
		}
		SREG = oldSREG;
	}
	return 0;
}





int put_string0 (char* p)
{
	if (*p == 0)
		return 0;

	short l = strlen (p)+1;

	while (l)
	{
		uint8_t c = *p++;
		put_char0 (c);
		l--;
	}

	return 0;
}





int get_c0(void)	// Non blocking, return status outside of char range
{
	int retc = AvrXPullFifo(Rx0Buf);
	return retc;
}




int get_char0(void)	// Blocks waiting for something
{
	return AvrXWaitPullFifo(Rx0Buf);
}
// We don't care if the buffer is full.  Just signal we got one.
// The task may attempt one extra time to get data out of an empyt
// buffer (getc(), but so what.  Eventually it will block waiting
// for another character to be received.




ISR (USART_RX_vect)
{
	AvrXPutFifo(Rx0Buf, UDR0);// This resets the Rx Interrupt and all error flags
}




ISR (USART_UDRE_vect)
{
	UCSR0B &= ~(1<<UDRIE0);			// Disable UDRE interrupt
}




ISR (USART_TX_vect)
{
	if (UCSR0A & (1<<UDRE0))
	{
		int c = AvrXPullFifo(Tx0Buf);	// Return -1 if empty (enables interrupts)
		if (c >= 0)						// Tricky tight code: only checking sign
			UDR0 = c;
	}
}

