/******************************************************
 *
 *	Control program for the PitSchu Rotary Encoder with stepper motor
 *	(c) Peter Schulten, Mlheim, Germany
 *	pit.schulten_(at)_arcor.de
 *
 *	Die unvernderte Wiedergabe und Verteilung dieses gesamten Sourcecodes
 *	in beliebiger Form ist gestattet, sofern obiger Hinweis erhalten bleibt.
 *
 * 	Ich stelle diesen Sourcecode kostenlos zur Verfgung und biete daher weder
 *	Support an noch garantiere ich fr seine Funktionsfhigkeit. Auerdem
 *	bernehme ich keine Haftung fr die Folgen seiner Nutzung.

 *	Der Sourcecode darf nur zu privaten Zwecken verwendet und modifiziert werden.
 *	Darber hinaus gehende Verwendung bedarf meiner Zustimmung.
 *
 *	History
 *	06.08.2014	PS		version 1 ready
 *
 */

#include "string.h"
#include "stdio.h"

#include "main.h"


// vars saved in EEPROM
short			avgCycles 		= 4000;
short			smoothCycles 	= 20;
short			hysteresis 		= 30;
short			breakTime		= TICKS_PER_SEC/4;		// 0.25 seconds
short			stepLimitL		= -140;
short			stepLimitR 		= 140;

short			booting = 0;

volatile		uint8_t	ticker;
volatile 		long time;
volatile short	breakCnt = 0;

volatile uint8_t ADCcycle = 0;
volatile short	stepADC[4];

short			stepAvg[2];
long			stepInt[2];
short			stepVal[2];
int8_t			stepState[2];		// 1 = high, 0 = undef, -1 = low
unsigned long	stepTS[2];			// timestamp of last edge (stepState)

volatile short	drehGeber = 0;		// the main step counter;

/********************************************************************************
 *
 * Initialize needed hardware components
 */

void IOinit (void) 
{
	CLKPR = _B1(CLKPCE);
	CLKPR = _B0(CLKPCE) | 0;		// set clock prescaler to zero

	PRR = _B1(PRTWI) | _B1(PRSPI);		// disable unused modules

	MCUCR = _B0(PUD);				// pull ups enabled

	DDR_LED	|= _B1(P_LED);
	LED_ON;

	DDR_DEB |= _B1(P_DEB0) | _B1(P_DEB1) | _B1(P_DEB2) |_B1(P_DEB3);
	DEB0_OFF;
	DEB1_OFF;
	DEB2_OFF;
	DEB3_OFF;

	DDR_BREAK |= _B1(P_BREAK_GND) | _B1(P_BREAK_VCC);
	BREAK_OFF;

	// Interrupts

	EIMSK = _B0(INT1) | _B0(INT0);		// external interrupts off
	PCICR = _B0(PCIE2) | _B0(PCIE1) | _B0(PCIE0);	// pin change irq off

	/* init ADC
		Free running mode = 1
		Int enabled = 1
		Prescale = 64 (giving 227KHz sample rate
		Reference voltage = AVCC
		Result right adjusted
	 */

	DIDR0 = _B1(P1_STEPPER) | _B1(P2_STEPPER) | _B1(P3_STEPPER) | _B1(P4_STEPPER);
	ADCSRA = _B1(ADEN) | _B1(ADATE) | _B1(ADIE) | _B1(ADPS2) |_B1(ADPS1) |_B1(ADPS0);	 // Freerun =1; Prescaler = 64
	ADCSRB = _B0(ADTS2) | _B0(ADTS1) | _B0(ADTS0);		// Freerun mode

	ADMUX = (_B0(REFS1) | _B1(REFS0) | _B0(ADLAR)) + 0; // Ref=AVCC; right adjusted; initiate measurment of chan 0

	// init timer 0 as sys ticker (ca. 900 Hz)
	TCNT0 = 0;
	OCR0A = (T0_CYCS - 1);
	TCCR0A = _B1(WGM01) | _B0(WGM00);
	TCCR0B = _B0(WGM02) | _B0(CS02) | _B1(CS01) | _B1(CS00);		// div 64 -> 230400 HZ
	TIMSK0 = _B1(OCF0A);

}




ISR (TIMER0_COMPA_vect)
{
	int8_t			stepDir = 0;

	ticker = 1;
	time++;

	DEB2_ON;

	if (breakCnt == 0)
	{
		if (stepState[0] && stepState[1] && stepTS[0] && stepTS[1])
		{
			if (stepState[0] > 0)
			{
				if (stepState[1] > 0)
				{
					if (stepTS[0] > stepTS[1])	// 2+ 1+
						stepDir = 1;
					else						// 1+ 2+
						stepDir = -1;
				}
				else
				{
					if (stepTS[0] > stepTS[1])	// 2- 1+
						stepDir = -1;
					else						// 1+ 2-
						stepDir = 1;
				}
			}
			else
			{
				if (stepState[1] > 0)
				{
					if (stepTS[0] > stepTS[1])	// 2+ 1-
						stepDir = -1;
					else						// 1- 2+
						stepDir = 1;
				}
				else
				{
					if (stepTS[0] > stepTS[1])	// 2- 1-
						stepDir = 1;
					else						// 1- 2-
						stepDir = -1;
				}
			}
			stepTS[0] = 0;
			stepTS[1] = 0;
		}
	}
	else				// break is activated; check for pulses on coil 1
	{
		if (stepState[0] && stepTS[0])		// got pulses; retrigger break
		{
			stepTS[0] = 0;
			breakCnt = breakTime;
		}
	}

	if (stepDir)
	{
		short s = drehGeber + stepDir;

		if (s < stepLimitL || s > stepLimitR)
		{
			breakCnt = breakTime;
		}
		else
		{
			drehGeber = s;
		}
	}

	if (breakCnt > 0)
	{
		breakCnt--;
		BREAK_ON;
		LED_ON;
	}
	else
	{
		BREAK_OFF;
		LED_OFF;
	}
	DEB2_OFF;
}




ISR (ADC_vect)
{
	DEB0_ON;
	// on entry, the MUX value we stored in last cycle will be sampled into ADC-S/H
	// measured value is from the second to last cycle

	register uint8_t i = (ADCcycle - 1) & 0x03;
	register short s;
	unsigned long ts =  ((time << 8) + TCNT0);


	if (breakCnt == 0 || ( i < 2 && breakCnt < (breakTime - 8)))
	{
		stepADC[i] = (short)((long)stepADC[i] * (smoothCycles-1) + (ADCL + (ADCH << 8))) / smoothCycles;	// get last ADC value

		if (i == 1 || i == 3)		// got new values for stepper coil 0/1 or 2/3
		{
			i >>= 1;

			stepVal[i] = (i == 0 ? stepADC[0] - stepADC[1] : stepADC[2] - stepADC[3]);	// diff of the 2 coil clamps
			s = stepVal[i] - stepAvg[i];

			stepInt[i] += stepVal[i];
			stepInt[i] -= stepAvg[i];
			stepAvg[i] = stepInt[i] / avgCycles;

			if (abs(s) <= hysteresis)
			{
				if (stepTS[i] > 0 && (ts - stepTS[i]) > T0_CNT_PER_SEC / 10)
				{
					stepState[i] = 0;
					stepTS[i] = 0;
				}
			}
			else
			{
				if (s > hysteresis && stepState[i] < 1)		// detect positive edge
				{
					stepState[i] = 1;
					stepTS[i] = ts;
				}
				else if (s < -1*hysteresis && stepState[i] >= 0)	// detect negative edge
				{
					stepState[i] = -1;
					stepTS[i] = ts;
				}
			}
		}
	}

	if (++ADCcycle > 3)
		ADCcycle = 0;

	ADMUX = ADCcycle | _B0(REFS1) | _B1(REFS0) | _B0(ADLAR); 	// Ref=AVCC; right adjusted

	DEB0_OFF;
}




int main (void)
{
	short dg = 0;
	int c;

	cli();
	IOinit ();				// init basics
	eeInit();
	sei ();

	// USART
#define T_BAUDRATE		(uint16_t)(((long)F_CPU / ((long)16 * 115200)) - 1)
	InitSerial0(T_BAUDRATE);
	_delay_ms(200);
	printf ("Hello world - this is pitschu\r\n\n");
	LED_OFF;

	ADCSRA |= _B1(ADSC);		// start ADC
	ADCcycle = 0;

	stepInt[0] = 0;
	stepInt[1] = 0;

	drehGeber = 0;
	_delay_ms (1000);		// let ADC sample some data to get avg values

	while (1)
	{
// example 1: send ascii bar graph to host
		if (dg != drehGeber)
		{
			char buffer[90];
			short i;

			dg = drehGeber;

			strcpy (buffer, "|.........|.........|.........|...-0+...|.........|.........|.........|         \r");

			i = 35 + dg / 4;
			buffer[i] = '^';
			put_string0(buffer);
		}

// example 2: send <drehgeber> value to host [+/-NNNN]; avoid printf() to save mem
#if 0
		if (dg != drehGeber)
		{
			char  	buffer[10];
			char   *cp = buffer+1;
			char   *cp1 = buffer +2;
			short d;

			dg = drehGeber;
			d = dg;

			strcpy (buffer, "\r       ");

			if (d < 0)
			{
				*cp++ = '-';
				d *= -1;
			}
			else
				*cp++ = '+';

			if (d == 0)
				*cp++ = '0';
			else
			{
				while (d > 0)
				{
					*cp++ = (uint8_t)'0' + (d % 10);
					d /= 10;
				}
			}
			cp--;

			while (cp > cp1)
			{
				uint8_t c = *cp1;
				*cp1++ = *cp;
				*cp-- = c;
			}

			put_string0(buffer);
		}
#endif

		if ((c = get_c0()) > 0)
		{
			// handle user input here; scan for new L/R limits or hyseresis values; .....
			// ..... here we just echo input to host
			put_c0 (c);
			if (c == '\r')
				put_c0('\n');
		}
	}

	return (0);
}

