/******************************************************
 *
 *	Control program for the PitSchu Rotary Encoder with stepper motor
 *	(c) Peter Schulten, Mlheim, Germany
 *	pit.schulten_(at)_arcor.de
 *
 *	Die unvernderte Wiedergabe und Verteilung dieses gesamten Sourcecodes
 *	in beliebiger Form ist gestattet, sofern obiger Hinweis erhalten bleibt.
 *
 * 	Ich stelle diesen Sourcecode kostenlos zur Verfgung und biete daher weder
 *	Support an noch garantiere ich fr seine Funktionsfhigkeit. Auerdem
 *	bernehme ich keine Haftung fr die Folgen seiner Nutzung.

 *	Der Sourcecode darf nur zu privaten Zwecken verwendet und modifiziert werden.
 *	Darber hinaus gehende Verwendung bedarf meiner Zustimmung.
 *
 *	History
 *	06.08.2014	PS		version 1 ready
 *
 */

#ifndef __MAIN_H__
#define __MAIN_H__

#ifndef F_CPU
#define F_CPU 14745600UL
#endif

#include <inttypes.h>
#include <string.h>
#include <math.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h> 
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "AvrXSerialIo.h"

// Fuse settings are:
//	L fuse:		0xF7
//	H fuse:		0xD4
//	E fuse:		0x03


 
//  logic defines

#define _B1(bit) 		(1 << (bit))  
#define _B0(bit) 		(0 << (bit))
#define true			1
#define	false			0 
#define MAX_INT         INT16_MAX
#define MAX_LONG        INT32_MAX

#define abs(a) ((a)>=0?(a):(-1*(a)))

/*
 *	Pin configuration:

	 1	PC 6	RESET		ISP-5
	 2	PD 0	RXD			RXD ro host (USB/serial)
	 3	PD 1	TXD			TXD from host (USB/serial)
	 4	PD 2				yellow LED (+)
	 5	PD 3	OC2B		stepper break GND
	 6	PD 4				stepper break VCC
	 7	VCC
	 8	GND
	 9	PB 6	Xtal1
	10	PB 7	Xtal2
	11	PD 5	OC0B	
	12	PD 6	OC0A
	13	PD 7
	14	PB 0	ICP


	15	PB 1			
	16	PB 2			
	17	PB 3	OC2A/MOSI	ISP-4
	18	PB 4	MISO		ISP-1
	19	PB 5	SCK			ISP-3
	20	AVCC
	21	AREF				100nf to GND
	22	AGND
	23	PC 0	ADC0		Stepper coil 1-A
	24	PC 1	ADC1		Stepper coil 1-B
	25	PC 2	ADC2		Stepper coil 2-A
	26	PC 3	ADC3		Stepper coil 2-B
	27	PC 4
	28	PC 5

*/

#define  T0_FREQ		(F_CPU)	// freq of timer 0 is 14.745600 MHz
#define  T0_DIVIDE		64
#define	 T0_CYCS		256		// timer 0 ticks until CTC event
#define  T0_CNT_PER_SEC	(T0_FREQ/T0_DIVIDE)
#define  TICKS_PER_SEC	((uint16_t)(T0_FREQ/T0_DIVIDE/T0_CYCS))	// clks for timer timeouts for 1 second


// hardware port defines

#define	PORT_LED	PORTD
#define	DDR_LED		DDRD
#define	P_LED		PD2

#define	PORT_DEB	PORTB
#define	DDR_DEB		DDRB
#define	P_DEB0		PB0
#define	P_DEB1		PB1
#define	P_DEB2		PB2
#define	P_DEB3		PB3

#define	PORT_BREAK	PORTD
#define	DDR_BREAK	DDRD
#define	P_BREAK_GND	PD3
#define	P_BREAK_VCC	PD4

#define	PORT_STEPPER	PORTC
#define	PIN_STEPPER	PINC
#define	DDR_STEPPER	DDRC
#define	P1_STEPPER	PC0
#define	P2_STEPPER	PC1
#define	P3_STEPPER	PC2
#define	P4_STEPPER	PC3


#define LOGGER(a,...)		printf_P(PSTR(a), ##__VA_ARGS__)
#define LED_ON				PORT_LED |= _B1(P_LED)
#define LED_OFF				PORT_LED &= ~(_B1(P_LED))
#define LED_TOGGLE			PORT_LED ^= _B1(P_LED)

#define DEB0_ON				PORT_DEB |= _B1(P_DEB0)
#define DEB0_OFF			PORT_DEB &= ~(_B1(P_DEB0))
#define DEB0_TOGGLE			PORT_DEB ^= _B1(P_DEB0)

#define DEB1_ON				PORT_DEB |= _B1(P_DEB1)
#define DEB1_OFF			PORT_DEB &= ~(_B1(P_DEB1))
#define DEB1_TOGGLE			PORT_DEB ^= _B1(P_DEB1)

#define DEB2_ON				PORT_DEB |= _B1(P_DEB2)
#define DEB2_OFF			PORT_DEB &= ~(_B1(P_DEB2))
#define DEB2_TOGGLE			PORT_DEB ^= _B1(P_DEB2)

#define DEB3_ON				PORT_DEB |= _B1(P_DEB3)
#define DEB3_OFF			PORT_DEB &= ~(_B1(P_DEB3))
#define DEB3_TOGGLE			PORT_DEB ^= _B1(P_DEB3)

#define BREAK_OFF			{PORT_BREAK |= _B1(P_BREAK_VCC); PORT_BREAK &= ~(_B1(P_BREAK_GND));}
#define BREAK_ON			{PORT_BREAK |= _B1(P_BREAK_GND); PORT_BREAK &= ~(_B1(P_BREAK_VCC));}
#define BREAK_IS_ON			(PORT_BREAK & _B1(P_BREAK_GND))


// EEPROM

typedef struct  {
	uint8_t		addr;	// Address in EEPROM; used as uint8_t*, ignore cc warnings
	void		*valP;	// points to variable which should be saved
	uint8_t		size;
} eeByte;

extern eeByte eeValues[];

extern void		eeInit(void);
extern void		eeUpdate (void);
extern void 	eeLoadSetup (void);

#endif


